@@ -1,15 +1,20 @@
-from collections.abc import Mapping
 from dataclasses import dataclass
-from typing import Tuple
+import sys
+from typing import Dict, Iterator, Tuple, Union
 import warnings
 
 from bigxml.handle_mgr import HandleMgr
 from bigxml.utils import extract_namespace_name
 
+if sys.version_info < (3, 9):  # pragma: no cover
+    from typing import Mapping
+else:  # pragma: no cover
+    from collections.abc import Mapping
 
-class XMLElementAttributes(Mapping):
-    def __init__(self, attributes):
-        self._items = {}  # key -> (alternatives, value)
+
+class XMLElementAttributes(Mapping[str, str]):
+    def __init__(self, attributes: Mapping[str, str]) -> None:
+        self._items: Dict[str, Tuple[int, str]] = {}  # key -> (alternatives, value)
         self._len = 0
         for key, value in attributes.items():
             namespace, name = extract_namespace_name(key)
@@ -22,7 +27,7 @@ class XMLElementAttributes(Mapping):
             else:
                 self._items[name] = (-1, value)
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> str:
         alternatives, value = self._items[key]
         if alternatives > 1:
             warnings.warn(
@@ -34,7 +39,7 @@ class XMLElementAttributes(Mapping):
             )
         return value
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[str]:
         for key, value in self._items.items():
             if key.startswith("{") and value[0] == -1:
                 if key.startswith("{}"):
@@ -42,17 +47,17 @@ class XMLElementAttributes(Mapping):
                 else:
                     yield key
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self._len
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"XMLElementAttributes({self})"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return str(dict(self))
 
 
-def _handler_get_text(node):
+def _handler_get_text(node: Union["XMLElement", "XMLText"]) -> Iterator[str]:
     if isinstance(node, XMLText):
         yield node.text
     elif isinstance(node, XMLElement):
@@ -65,14 +70,14 @@ def _handler_get_text(node):
 class XMLElement(HandleMgr):
     name: str
     attributes: XMLElementAttributes
-    parents: Tuple["XMLElement"]
+    parents: Tuple["XMLElement", ...]
     namespace: str = ""
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         if not self.namespace:
             self.namespace, self.name = extract_namespace_name(self.name)
 
-    def __str__(self):
+    def __str__(self) -> str:
         parts = []
         if self.namespace:
             parts.append(f"{{{self.namespace}}}{self.name}")
@@ -85,7 +90,7 @@ class XMLElement(HandleMgr):
         return f"XMLElement({', '.join(parts)})"
 
     @property
-    def text(self):
+    def text(self) -> str:
         output = ""
         last_ends_with_space = False
         for text in self.iter_from(_handler_get_text):
@@ -102,12 +107,12 @@ class XMLElement(HandleMgr):
 @dataclass
 class XMLText:
     text: str
-    parents: Tuple["XMLElement"]
+    parents: Tuple[XMLElement, ...]
 
     # classname attribute name to be easily switched on with XMLElement
     name = "\0text"  # \0 makes sure it is an invalid element name
 
-    def __str__(self):
+    def __str__(self) -> str:
         parts = [repr(self.text)]
         if self.parents:
             parts.append(f"parents={'>'.join(node.name for node in self.parents)}")
