@@ -1,10 +1,12 @@
+from typing import Callable, Iterator
+
 import pytest
 
-from bigxml import Parser, xml_handle_text
+from bigxml import Parser, XMLText, xml_handle_text
 
 
 @pytest.fixture
-def ram_usage():
+def ram_usage() -> Iterator[Callable[[], float]]:
     try:
         import tracemalloc  # pylint: disable=import-outside-toplevel
     except ImportError:  # e.g. PyPy
@@ -17,8 +19,8 @@ def ram_usage():
         tracemalloc.stop()
 
 
-def big_stream(ram_used):
-    ram_limit = None
+def big_stream(ram_used: Callable[[], float]) -> Iterator[bytes]:
+    ram_limit: float | None = None
 
     yield b"<root>\n"
 
@@ -47,10 +49,10 @@ def big_stream(ram_used):
 
 def test_with_handler(
     # pylint: disable=redefined-outer-name
-    ram_usage,
-):
+    ram_usage: Callable[[], float],
+) -> None:
     @xml_handle_text("root", "entry", "nb")
-    def handler(node):
+    def handler(node: XMLText) -> Iterator[int]:
         yield int(node.text)
 
     items = Parser(big_stream(ram_usage)).iter_from(handler)
@@ -61,7 +63,7 @@ def test_with_handler(
 
 def test_no_handler(
     # pylint: disable=redefined-outer-name
-    ram_usage,
-):
-    items = Parser(big_stream(ram_usage)).iter_from()
+    ram_usage: Callable[[], float],
+) -> None:
+    items: Iterator[object] = Parser(big_stream(ram_usage)).iter_from()
     assert not list(items)
