@@ -1,15 +1,16 @@
-from defusedxml import EntitiesForbidden
+from typing import Iterator
+
 import pytest
 
-from bigxml import Parser
+from bigxml import BigXmlException, Parser, XMLText
 
 
-def handler_get_text(node):
+def handler_get_text(node: XMLText) -> Iterator[str]:
     yield node.text
 
 
 @pytest.mark.parametrize(
-    "xml",
+    "xml, msg",
     (
         pytest.param(
             (
@@ -18,6 +19,7 @@ def handler_get_text(node):
                 b"]>\n"
                 b"<root>&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;&a;</root>\n"
             ),
+            "Entity definition is forbidden",
             id="one entity (1kb)",
         ),
         pytest.param(
@@ -31,6 +33,7 @@ def handler_get_text(node):
                 b"]>\n"
                 b"<root>&e;</root>\n"
             ),
+            "Entity definition is forbidden",
             id="bomb (5Mb)",
         ),
         pytest.param(
@@ -47,6 +50,7 @@ def handler_get_text(node):
                 b"]>\n"
                 b"<root>&h;</root>\n"
             ),
+            "Entity definition is forbidden",
             id="bomb (33Gb)",
         ),
         pytest.param(
@@ -57,6 +61,7 @@ def handler_get_text(node):
                 b"]>\n"
                 b"<root>&a;</root>\n"
             ),
+            "Entity definition is forbidden",
             id="recursive",
         ),
         pytest.param(
@@ -66,6 +71,7 @@ def handler_get_text(node):
                 b"]>\n"
                 b"<root>&a;</root>\n"
             ),
+            "Entity definition is forbidden",
             id="resolution (file)",
         ),
         pytest.param(
@@ -75,6 +81,7 @@ def handler_get_text(node):
                 b"]>\n"
                 b"<root>&a;</root>\n"
             ),
+            "Entity definition is forbidden",
             id="resolution (http)",
         ),
         pytest.param(
@@ -84,10 +91,13 @@ def handler_get_text(node):
                 b"]>\n"
                 b"<root>&a;</root>\n"
             ),
+            "Entity definition is forbidden",
             id="resolution (expect)",
         ),
     ),
 )
-def test_external_entities(xml):
-    with pytest.raises(EntitiesForbidden):
+def test_external_entities(xml: bytes, msg: str) -> None:
+    with pytest.raises(BigXmlException) as exc_info:
         Parser(xml).return_from(handler_get_text)
+    assert str(exc_info.value) == msg
+    assert exc_info.value.security
