@@ -1,3 +1,4 @@
+from typing import Callable, Iterator, Union
 from unittest.mock import Mock
 
 import pytest
@@ -5,15 +6,19 @@ import pytest
 from bigxml.nodes import XMLElement, XMLText
 
 
-def create_text(text):
+def create_text(text: str) -> XMLText:
     return XMLText(text, ())  # don't care about parents
 
 
-def create_element(*children):
-    node = XMLElement("foo", {}, ())  # don't care about parents
+def create_element(*children: Union[XMLElement, XMLText]) -> XMLElement:
+    node = XMLElement("foo", Mock(), ())  # don't care about parents
     handle = Mock()
 
-    def side_effect(handler):
+    def side_effect(
+        handler: Callable[
+            [Union[XMLElement, XMLText]], Iterator[Union[XMLElement, XMLText]]
+        ]
+    ) -> Iterator[Union[XMLElement, XMLText]]:
         for child in children:
             yield from handler(child)
 
@@ -23,7 +28,7 @@ def create_element(*children):
 
 
 @pytest.mark.parametrize("text", ("abc", "  \n  abc \n  "))
-def test_element_get_text_direct(text):
+def test_element_get_text_direct(text: str) -> None:
     node = create_element(create_text(text))
     assert node.text == "abc"
 
@@ -41,7 +46,7 @@ def test_element_get_text_direct(text):
         ("a ", "", " c", "a c"),
     ),
 )
-def test_element_get_text_nested(left, middle, right, exp):
+def test_element_get_text_nested(left: str, middle: str, right: str, exp: str) -> None:
     node = create_element(
         create_text(left),
         create_element(create_text(middle)),
